"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadNS = void 0;
const ytdl_core_muxer_1 = __importDefault(require("ytdl-core-muxer"));
const fluent_ffmpeg_1 = __importDefault(require("fluent-ffmpeg"));
const fs_1 = __importDefault(require("fs"));
const promises_1 = require("fs/promises");
const os_1 = require("os");
const nanostores_cjs_1 = require("nanostores-cjs");
const repl_ns_1 = require("repl-ns");
const duration_1 = require("../src/duration");
exports.downloadNS = (0, repl_ns_1.ns)('download', {
    async onDownload(state) {
        (0, exports.downloadNS)().renderInProgress();
        try {
            const temporaryFilePath = await (0, exports.downloadNS)().createTempFilePath();
            (0, exports.downloadNS)()
                .youtubeDownload(state.url)
                .on('error', err => (0, exports.downloadNS)().renderErr(err))
                .pipe((0, exports.downloadNS)()
                .writeTempFile(temporaryFilePath)
                .on('error', err => (0, exports.downloadNS)().renderErr(err)))
                .on('finish', async () => {
                try {
                    if (state.startTime || state.endTime || state.resolution !== 'highest') {
                        let ffmpegStream = (0, exports.downloadNS)().createFfmpeg(temporaryFilePath);
                        if (state.startTime) {
                            ffmpegStream = ffmpegStream.setStartTime(state.startTime);
                        }
                        if (state.endTime) {
                            ffmpegStream = ffmpegStream.setDuration((0, duration_1.durationNS)().calcDuration(state));
                        }
                        if (state.resolution !== 'highest') {
                            ffmpegStream = ffmpegStream.size(`?x${state.resolution}`);
                        }
                        ffmpegStream
                            .saveToFile(state.path)
                            .on('error', err => (0, exports.downloadNS)().renderErr(err))
                            .on('end', () => (0, exports.downloadNS)().renderComplete());
                    }
                    else {
                        await (0, exports.downloadNS)().cpFile(temporaryFilePath, state.path);
                        (0, exports.downloadNS)().renderComplete();
                    }
                }
                catch (err) {
                    (0, exports.downloadNS)().renderErr(err);
                }
            });
        }
        catch (err) {
            (0, exports.downloadNS)().renderErr(err);
        }
    },
    async createTempFilePath() {
        return `${await (0, promises_1.mkdtemp)(`${(0, os_1.tmpdir)()}/ytdl-tui-`)}/video.mp4`;
    },
    youtubeDownload(url) {
        return (0, ytdl_core_muxer_1.default)(url);
    },
    writeTempFile(path) {
        return fs_1.default.createWriteStream(path);
    },
    createFfmpeg(path) {
        return (0, fluent_ffmpeg_1.default)(path);
    },
    async cpFile(from, to) {
        await (0, promises_1.copyFile)(from, to);
    },
    renderComplete() {
        (0, exports.downloadNS)().downloadInProgress$.set(false);
        (0, exports.downloadNS)().downloaded$.set(true);
        (0, exports.downloadNS)().downloadError$.set(null);
    },
    renderErr(err) {
        (0, exports.downloadNS)().downloadInProgress$.set(false);
        (0, exports.downloadNS)().downloaded$.set(false);
        (0, exports.downloadNS)().downloadError$.set(err);
    },
    renderInProgress() {
        (0, exports.downloadNS)().downloadError$.set(null);
        (0, exports.downloadNS)().downloaded$.set(false);
        (0, exports.downloadNS)().downloadInProgress$.set(true);
    },
    downloaded$: (0, nanostores_cjs_1.atom)(false),
    downloadInProgress$: (0, nanostores_cjs_1.atom)(false),
    downloadError$: (0, nanostores_cjs_1.atom)(null),
});
